@@ -23,10 +23,13 @@ package Lintian::triggers;
 use strict;
 use warnings;
 
+use Moo;
+
 use Lintian::Data;
-use Lintian::Tags qw(tag);
 use Lintian::Util qw(internal_error strip);
 
+with('Lintian::Check');
+
 sub _parse_trigger_types {
     my ($key, $val) = @_;
     my %values;
@@ -49,7 +52,10 @@ my $TRIGGER_TYPES = Lintian::Data->new('triggers/trigger-types',
     qr/\s*\Q=>\E\s*/, \&_parse_trigger_types);
 
 sub always {
-    my (undef, undef, $info) = @_;
+    my ($self) = @_;
+
+    my $info = $self->info;
+
     my $triggers_file = $info->control_index('triggers');
     return if not $triggers_file or not $triggers_file->is_open_ok;
     my $fd = $triggers_file->open;
@@ -60,16 +66,16 @@ sub always {
         my ($trigger_type, $arg) = split(m/\s++/, $line, 2);
         my $trigger_info = $TRIGGER_TYPES->value($trigger_type);
         if (not $trigger_info) {
-            tag 'unknown-trigger', $line, "(line $.)";
+            $self->tag('unknown-trigger', $line, "(line $.)");
             next;
         }
         if ($trigger_info->{'implicit-await'}) {
-            tag 'uses-implicit-await-trigger', $line, "(line $.)";
+            $self->tag('uses-implicit-await-trigger', $line, "(line $.)");
         }
         if (defined(my $prev_info = $seen_triggers{$arg})) {
             my ($prev_line, $prev_line_no) = @{$prev_info};
-            tag 'repeated-trigger-name', $line, "(line $.)", 'vs', $prev_line,
-              "(line $prev_line_no)";
+            $self->tag('repeated-trigger-name', $line, "(line $.)", 'vs',
+                $prev_line,"(line $prev_line_no)");
             next;
         }
         $seen_triggers{$arg} = [$line, $.];
